"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const AbstractIndex_1 = require("../AbstractIndex");
const constants_1 = require("../../../common-libs/constants");
const constants = require('./../../../constants');
const indexer = require('../../../indexer').Indexer;
class CIndexDAL extends AbstractIndex_1.AbstractIndex {
    constructor(driver) {
        super(driver, 'c_index', 
        // PK fields
        ['op', 'issuer', 'receiver', 'written_on'], 
        // Fields
        [
            'op',
            'issuer',
            'receiver',
            'created_on',
            'written_on',
            'writtenOn',
            'sig',
            'expires_on',
            'expired_on',
            'chainable_on',
            'from_wid',
            'to_wid'
        ], 
        // Arrays
        [], 
        // Booleans
        [], 
        // BigIntegers
        [], 
        // Transient
        []);
    }
    async init() {
        await this.exec('BEGIN;' +
            'CREATE TABLE IF NOT EXISTS ' + this.table + ' (' +
            'op VARCHAR(10) NOT NULL,' +
            'issuer VARCHAR(50) NOT NULL,' +
            'receiver VARCHAR(50) NOT NULL,' +
            'created_on VARCHAR(80) NOT NULL,' +
            'written_on VARCHAR(80) NOT NULL,' +
            'sig VARCHAR(100) NULL,' +
            'expires_on INTEGER NULL,' +
            'expired_on INTEGER NULL,' +
            'chainable_on INTEGER NULL,' +
            'from_wid INTEGER NULL,' +
            'to_wid INTEGER NULL,' +
            'PRIMARY KEY (op,issuer,receiver,written_on)' +
            ');' +
            'CREATE INDEX IF NOT EXISTS idx_cindex_issuer ON c_index (issuer);' +
            'CREATE INDEX IF NOT EXISTS idx_cindex_receiver ON c_index (receiver);' +
            'CREATE INDEX IF NOT EXISTS idx_cindex_chainable_on ON c_index (chainable_on);' +
            'COMMIT;');
    }
    async reducablesFrom(from) {
        const reducables = await this.query('SELECT * FROM ' + this.table + ' WHERE issuer = ? ORDER BY CAST(written_on as integer) ASC', [from]);
        return indexer.DUP_HELPERS.reduceBy(reducables, ['issuer', 'receiver', 'created_on']);
    }
    async trimExpiredCerts(belowNumber) {
        const toDelete = await this.query('SELECT * FROM ' + this.table + ' WHERE expired_on > ? AND CAST(written_on as int) < ?', [0, belowNumber]);
        for (const row of toDelete) {
            await this.exec("DELETE FROM " + this.table + " " +
                "WHERE issuer like '" + row.issuer + "' " +
                "AND receiver = '" + row.receiver + "' " +
                "AND created_on like '" + row.created_on + "'");
        }
    }
    getWrittenOn(blockstamp) {
        return this.sqlFind({ written_on: blockstamp });
    }
    findExpired(medianTime) {
        return this.query('SELECT * FROM ' + this.table + ' c1 WHERE expires_on <= ? ' +
            'AND NOT EXISTS (' +
            ' SELECT * FROM c_index c2' +
            ' WHERE c1.issuer = c2.issuer' +
            ' AND c1.receiver = c2.receiver' +
            ' AND c1.created_on = c2.created_on' +
            ' AND c2.op = ?' +
            ')', [medianTime, constants_1.CommonConstants.IDX_UPDATE]);
    }
    getValidLinksTo(receiver) {
        return this.query('SELECT * FROM ' + this.table + ' c1 ' +
            'WHERE c1.receiver = ? ' +
            'AND c1.expired_on = 0 ' +
            'AND NOT EXISTS (' +
            ' SELECT * FROM c_index c2' +
            ' WHERE c1.issuer = c2.issuer' +
            ' AND c1.receiver = c2.receiver' +
            ' AND c1.created_on = c2.created_on' +
            ' AND c2.op = ?' +
            ')', [receiver, constants_1.CommonConstants.IDX_UPDATE]);
    }
    getValidLinksFrom(issuer) {
        return this.query('SELECT * FROM ' + this.table + ' c1 ' +
            'WHERE c1.issuer = ? ' +
            'AND c1.expired_on = 0 ' +
            'AND NOT EXISTS (' +
            ' SELECT * FROM c_index c2' +
            ' WHERE c1.issuer = c2.issuer' +
            ' AND c1.receiver = c2.receiver' +
            ' AND c1.created_on = c2.created_on' +
            ' AND c2.op = ?' +
            ')', [issuer, constants_1.CommonConstants.IDX_UPDATE]);
    }
    async existsNonReplayableLink(issuer, receiver) {
        const results = await this.query('SELECT * FROM ' + this.table + ' c1 ' +
            'WHERE c1.issuer = ? ' +
            'AND c1.receiver = ? ' +
            'AND NOT EXISTS (' +
            ' SELECT * FROM c_index c2' +
            ' WHERE c1.issuer = c2.issuer' +
            ' AND c1.receiver = c2.receiver' +
            ' AND c1.created_on = c2.created_on' +
            ' AND c2.op = ?' +
            ')', [issuer, receiver, constants_1.CommonConstants.IDX_UPDATE]);
        return results.length > 0;
    }
    removeBlock(blockstamp) {
        return this.exec('DELETE FROM ' + this.table + ' WHERE written_on = \'' + blockstamp + '\'');
    }
}
exports.CIndexDAL = CIndexDAL;
//# sourceMappingURL=CIndexDAL.js.map