import { BlockDTO } from "../dto/BlockDTO";
import { ConfDTO } from "../dto/ConfDTO";
import { IndexEntry } from "../indexer";
import { TransactionDTO } from "../dto/TransactionDTO";
import { DBBlock } from "../db/DBBlock";
export declare const LOCAL_RULES_FUNCTIONS: {
    checkParameters: (block: BlockDTO) => Promise<boolean>;
    isProofOfWorkCorrect: (block: BlockDTO) => boolean;
    checkProofOfWork: (block: BlockDTO) => Promise<boolean>;
    checkInnerHash: (block: BlockDTO) => Promise<boolean>;
    checkPreviousHash: (block: BlockDTO) => Promise<boolean>;
    checkPreviousIssuer: (block: BlockDTO) => Promise<boolean>;
    checkUnitBase: (block: BlockDTO) => Promise<boolean>;
    checkBlockSignature: (block: BlockDTO) => Promise<boolean>;
    checkBlockTimes: (block: BlockDTO, conf: ConfDTO) => Promise<boolean>;
    checkIdentitiesSignature: (block: BlockDTO) => Promise<boolean>;
    checkIdentitiesUserIDConflict: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkIdentitiesPubkeyConflict: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkIdentitiesMatchJoin: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkRevokedAreExcluded: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkRevokedUnicity: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkMembershipUnicity: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkMembershipsSignature: (block: BlockDTO) => Promise<boolean>;
    checkPubkeyUnicity: (block: BlockDTO) => Promise<boolean>;
    checkCertificationOneByIssuer: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkCertificationUnicity: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkCertificationIsntForLeaverOrExcluded: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
    checkTxVersion: (block: BlockDTO) => Promise<boolean>;
    checkTxLen: (block: BlockDTO) => Promise<boolean>;
    checkTxIssuers: (block: BlockDTO) => Promise<boolean>;
    checkTxSources: (block: BlockDTO) => Promise<boolean>;
    checkTxAmounts: (block: BlockDTO) => Promise<void>;
    checkTxRecipients: (block: BlockDTO) => Promise<boolean>;
    checkTxSignature: (block: BlockDTO) => Promise<boolean>;
    checkMaxTransactionChainingDepth: (block: BlockDTO, conf: ConfDTO, index: IndexEntry[]) => Promise<boolean>;
};
export interface SindexShortEntry {
    op: string;
    identifier: string;
    pos: number;
    tx: string | null;
}
export declare const LOCAL_RULES_HELPERS: {
    maxAcceleration: (conf: ConfDTO) => any;
    checkSingleMembershipSignature: (ms: any) => any;
    checkBunchOfTransactions: (transactions: TransactionDTO[], conf: ConfDTO, options?: {
        dontCareAboutChaining?: boolean | undefined;
    } | undefined) => Promise<void>;
    getTransactionDepth: (txHash: string, sindex: SindexShortEntry[], localDepth?: number) => number;
    getMaxTransactionDepth: (sindex: SindexShortEntry[]) => number;
    checkSingleTransactionLocally: (tx: any, conf: ConfDTO) => Promise<void>;
    checkTxAmountsValidity: (tx: TransactionDTO) => void;
    getMaxPossibleVersionNumber: (current: DBBlock) => Promise<number>;
};
