"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const SQLiteDriver_1 = require("../dal/drivers/SQLiteDriver");
const CFSCore_1 = require("../dal/fileDALs/CFSCore");
const wot_1 = require("../wot");
const opts = require('optimist').argv;
const path = require('path');
const qfs = require('q-io/fs');
const fs = require('fs');
const DEFAULT_DOMAIN = "duniter_default";
const DEFAULT_HOME = (process.platform == 'win32' ? process.env.USERPROFILE : process.env.HOME) + '/.config/duniter/';
const getLogsPath = (profile, directory = null) => path.join(getHomePath(profile, directory), 'duniter.log');
const getHomePath = (profile, directory = null) => path.normalize(getUserHome(directory) + '/') + getDomain(profile);
const getUserHome = (directory = null) => (directory || DEFAULT_HOME);
const getDomain = (profile = null) => (profile || DEFAULT_DOMAIN);
const dir = module.exports = {
    INSTANCE_NAME: getDomain(opts.mdb),
    INSTANCE_HOME: getHomePath(opts.mdb, opts.home),
    INSTANCE_HOMELOG_FILE: getLogsPath(opts.mdb, opts.home),
    DUNITER_DB_NAME: 'duniter',
    WOTB_FILE: 'wotb.bin',
    getHome: (profile = null, directory = null) => getHomePath(profile, directory),
    getHomeFS: async (isMemory, theHome, makeTree = true) => {
        const home = theHome || dir.getHome();
        const params = {
            home: home
        };
        if (isMemory) {
            params.fs = require('q-io/fs-mock')({});
        }
        else {
            params.fs = qfs;
        }
        if (makeTree) {
            await params.fs.makeTree(home);
        }
        return params;
    },
    getHomeParams: async (isMemory, theHome) => {
        const params = await dir.getHomeFS(isMemory, theHome);
        const home = params.home;
        if (isMemory) {
            params.dbf = () => new SQLiteDriver_1.SQLiteDriver(':memory:');
            params.wotb = wot_1.WoTBObject.memoryInstance();
        }
        else {
            const sqlitePath = path.join(home, dir.DUNITER_DB_NAME + '.db');
            params.dbf = () => new SQLiteDriver_1.SQLiteDriver(sqlitePath);
            const wotbFilePath = path.join(home, dir.WOTB_FILE);
            let existsFile = await qfs.exists(wotbFilePath);
            if (!existsFile) {
                fs.closeSync(fs.openSync(wotbFilePath, 'w'));
            }
            params.wotb = wot_1.WoTBObject.fileInstance(wotbFilePath);
        }
        return params;
    },
    createHomeIfNotExists: async (fileSystem, theHome) => {
        const fsHandler = new CFSCore_1.CFSCore(theHome, fileSystem);
        return fsHandler.makeTree('');
    }
};
//# sourceMappingURL=directory.js.map