"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
class WS2PReqMapperByServer {
    constructor(server) {
        this.server = server;
    }
    async getCurrent() {
        return this.server.BlockchainService.current();
    }
    getBlock(number) {
        return this.server.dal.getBlock(number);
    }
    async getBlocks(count, from) {
        if (count > 5000) {
            throw 'Count is too high';
        }
        const current = await this.server.dal.getCurrentBlockOrNull();
        count = Math.min(current.number - from + 1, count);
        if (!current || current.number < from) {
            return [];
        }
        return this.server.dal.getBlocksBetween(from, from + count - 1);
    }
    async getRequirementsOfPending(minsig) {
        let identities = await this.server.dal.idtyDAL.query('SELECT i.*, count(c.sig) as nbSig ' +
            'FROM idty i, cert c ' +
            'WHERE c.target = i.hash group by i.hash having nbSig >= ?', minsig);
        const members = (await this.server.dal.idtyDAL.query('SELECT i.*, count(c.sig) as nbSig ' +
            'FROM i_index i, cert c ' +
            'WHERE c.`to` = i.pub group by i.pub having nbSig >= ?', minsig)).map((i) => {
            return {
                hash: i.hash || "",
                member: i.member || false,
                wasMember: i.wasMember || false,
                pubkey: i.pub,
                uid: i.uid || "",
                buid: i.created_on || "",
                sig: i.sig || "",
                revocation_sig: "",
                revoked: false,
                revoked_on: 0
            };
        });
        identities = identities.concat(members);
        const all = await this.server.BlockchainService.requirementsOfIdentities(identities, false);
        return {
            identities: all
        };
    }
}
exports.WS2PReqMapperByServer = WS2PReqMapperByServer;
//# sourceMappingURL=WS2PReqMapperByServer.js.map