"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("../../../server");
const BlockDTO_1 = require("../../../app/lib/dto/BlockDTO");
const IdentityDTO_1 = require("../../../app/lib/dto/IdentityDTO");
const PeerDTO_1 = require("../../../app/lib/dto/PeerDTO");
const network_1 = require("../../../app/modules/bma/lib/network");
const ConfDTO_1 = require("../../../app/lib/dto/ConfDTO");
const WS2PConnection_1 = require("../../../app/modules/ws2p/lib/WS2PConnection");
const WS2PCluster_1 = require("../../../app/modules/ws2p/lib/WS2PCluster");
const WS2PServerMessageHandler_1 = require("../../../app/modules/ws2p/lib/interface/WS2PServerMessageHandler");
const TestUser_1 = require("./TestUser");
const router_1 = require("../../../app/modules/router");
const assert = require('assert');
const _ = require('underscore');
const rp = require('request-promise');
const es = require('event-stream');
const WebSocketServer = require('ws').Server;
const httpTest = require('../tools/http');
const sync = require('../tools/sync');
const commit = require('../tools/commit');
const until = require('../tools/until');
const bma = require('../../../app/modules/bma').BmaDependency.duniter.methods.bma;
const logger = require('../../../app/lib/logger').NewLogger('toolbox');
require('../../../app/modules/bma').BmaDependency.duniter.methods.noLimit(); // Disables the HTTP limiter
const MEMORY_MODE = true;
const CURRENCY_NAME = 'duniter_unit_test_currency';
const HOST = '127.0.0.1';
let PORT = 10000;
exports.getNewTestingPort = () => {
    return PORT++;
};
exports.shouldFail = async (promise, message = null) => {
    try {
        await promise;
        throw '{ "message": "Should have thrown an error" }';
    }
    catch (e) {
        let err = e;
        if (typeof e === "string") {
            err = JSON.parse(e);
        }
        err.should.have.property('message').equal(message);
    }
};
exports.assertThrows = async (promise, message = null) => {
    try {
        await promise;
        throw "Should have thrown";
    }
    catch (e) {
        if (e === "Should have thrown") {
            throw e;
        }
        assert.equal(e, message);
    }
};
exports.simpleUser = (uid, keyring, server) => {
    return new TestUser_1.TestUser(uid, keyring, { server });
};
exports.simpleNetworkOf2NodesAnd2Users = async (options) => {
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    const s1 = exports.NewTestingServer(_.extend({ pair: catKeyring }, options || {}));
    const s2 = exports.NewTestingServer(_.extend({ pair: tacKeyring }, options || {}));
    const cat = new TestUser_1.TestUser('cat', catKeyring, { server: s1 });
    const tac = new TestUser_1.TestUser('tac', tacKeyring, { server: s1 });
    await s1.initDalBmaConnections();
    await s2.initDalBmaConnections();
    await s2.sharePeeringWith(s1);
    // await s2.post('/network/peering/peers', await s1.get('/network/peering'));
    // await s1.submitPeerP(await s2.get('/network/peering'));
    await cat.createIdentity();
    await tac.createIdentity();
    await cat.cert(tac);
    await tac.cert(cat);
    await cat.join();
    await tac.join();
    // Each server forwards to each other
    router_1.RouterDependency.duniter.methods.routeToNetwork(s1._server);
    router_1.RouterDependency.duniter.methods.routeToNetwork(s2._server);
    return { s1, s2, cat, tac };
};
exports.simpleNodeWith2Users = async (options) => {
    const catKeyring = { pub: 'HgTTJLAQ5sqfknMq7yLPZbehtuLSsKj9CxWN7k8QvYJd', sec: '51w4fEShBk1jCMauWu4mLpmDVfHksKmWcygpxriqCEZizbtERA6de4STKRkQBpxmMUwsKXRjSzuQ8ECwmqN1u2DP' };
    const tacKeyring = { pub: '2LvDg21dVXvetTD9GdkPLURavLYEqP3whauvPWX4c2qc', sec: '2HuRLWgKgED1bVio1tdpeXrf7zuUszv1yPHDsDj7kcMC4rVSN9RC58ogjtKNfTbH1eFz7rn38U1PywNs3m6Q7UxE' };
    const s1 = exports.NewTestingServer(_.extend({ pair: catKeyring }, options || {}));
    const cat = new TestUser_1.TestUser('cat', catKeyring, { server: s1 });
    const tac = new TestUser_1.TestUser('tac', tacKeyring, { server: s1 });
    await s1.initDalBmaConnections();
    await cat.createIdentity();
    await tac.createIdentity();
    await cat.cert(tac);
    await tac.cert(cat);
    await cat.join();
    await tac.join();
    return { s1, cat, tac };
};
exports.simpleNodeWith2otherUsers = async (options) => {
    const ticKeyring = { pub: 'DNann1Lh55eZMEDXeYt59bzHbA3NJR46DeQYCS2qQdLV', sec: '468Q1XtTq7h84NorZdWBZFJrGkB18CbmbHr9tkp9snt5GiERP7ySs3wM8myLccbAAGejgMRC9rqnXuW3iAfZACm7' };
    const tocKeyring = { pub: 'DKpQPUL4ckzXYdnDRvCRKAm1gNvSdmAXnTrJZ7LvM5Qo', sec: '64EYRvdPpTfLGGmaX5nijLXRqWXaVz8r1Z1GtaahXwVSJGQRn7tqkxLb288zwSYzELMEG5ZhXSBYSxsTsz1m9y8F' };
    const s1 = exports.NewTestingServer(_.extend({ pair: ticKeyring }, options || {}));
    const tic = new TestUser_1.TestUser('cat', ticKeyring, { server: s1 });
    const toc = new TestUser_1.TestUser('tac', tocKeyring, { server: s1 });
    await s1.initDalBmaConnections();
    await tic.createIdentity();
    await toc.createIdentity();
    await tic.cert(toc);
    await toc.cert(tic);
    await tic.join();
    await toc.join();
    return { s1, tic, toc };
};
exports.createUser = async (uid, pub, sec, defaultServer) => {
    const keyring = { pub: pub, sec: sec };
    return new TestUser_1.TestUser(uid, keyring, { server: defaultServer });
};
exports.fakeSyncServer = async (readBlocksMethod, readParticularBlockMethod, onPeersRequested) => {
    const host = HOST;
    const port = PORT++;
    // Meaningful variables
    const NO_HTTP_LOGS = false;
    const NO_STATIC_PATH = null;
    // A fake HTTP limiter with no limit at all
    const noLimit = {
        canAnswerNow: () => true,
        processRequest: () => { }
    };
    const fakeServer = await network_1.Network.createServersAndListen("Fake Duniter Server", new server_1.Server("", true, ConfDTO_1.ConfDTO.mock()), [{
            ip: host,
            port: port
        }], NO_HTTP_LOGS, logger, NO_STATIC_PATH, (app, httpMethods) => {
        // Mock BMA method for sync mocking
        httpMethods.httpGET('/network/peering', async () => {
            return {
                endpoints: [['BASIC_MERKLED_API', host, port].join(' ')]
            };
        }, noLimit);
        // Mock BMA method for sync mocking
        httpMethods.httpGET('/network/peering/peers', onPeersRequested, noLimit);
        // Another mock BMA method for sync mocking
        httpMethods.httpGET('/blockchain/blocks/:count/:from', (req) => {
            // What do we do on /blockchain/blocks request
            let count = parseInt(req.params.count);
            let from = parseInt(req.params.from);
            return readBlocksMethod(count, from);
        }, noLimit);
        // Another mock BMA method for sync mocking
        httpMethods.httpGET('/blockchain/block/:number', (req) => {
            // What do we do on /blockchain/blocks request
            let number = parseInt(req.params.number);
            return readParticularBlockMethod(number);
        }, noLimit);
    }, null);
    await fakeServer.openConnections();
    return {
        host: host,
        port: port
    };
};
/**
 * Creates a new memory duniter server for Unit Test purposes.
 * @param conf
 */
exports.server = (conf) => exports.NewTestingServer(conf);
exports.simpleTestingServer = (conf) => exports.NewTestingServer(conf);
exports.NewTestingServer = (conf) => {
    const host = conf.host || HOST;
    const port = conf.port || PORT++;
    const commonConf = {
        nobma: false,
        bmaWithCrawler: true,
        port: port,
        ipv4: host,
        remoteipv4: host,
        currency: conf.currency || CURRENCY_NAME,
        httpLogs: true,
        forksize: conf.forksize || 3
    };
    if (conf.sigQty === undefined) {
        conf.sigQty = 1;
    }
    // Disable UPnP during tests
    if (!conf.ws2p) {
        conf.ws2p = { upnp: false };
    }
    const server = new server_1.Server('~/.config/duniter/' + (conf.homename || 'dev_unit_tests'), conf.memory !== undefined ? conf.memory : MEMORY_MODE, _.extend(conf, commonConf));
    return new TestingServer(port, server);
};
exports.serverWaitBlock = async (server, number) => {
    await new Promise((res) => {
        const interval = setInterval(async () => {
            const current = await server.dal.getCurrentBlockOrNull();
            if (current && current.number == number) {
                res();
                clearInterval(interval);
            }
        }, 1);
    });
};
exports.waitToHaveBlock = async (server, number) => {
    return exports.serverWaitBlock(server, number);
};
exports.waitForkResolution = async (server, number) => {
    await new Promise(res => {
        server.pipe(es.mapSync((e) => {
            if (e.bcEvent === 'switched' && e.block.number === number) {
                res();
            }
            return e;
        }));
    });
};
exports.waitForkWS2PConnection = async (server, pubkey) => {
    await new Promise(res => {
        server.pipe(es.mapSync((e) => {
            if (e.ws2p === 'connected' && e.to.pubkey === pubkey) {
                res();
            }
            return e;
        }));
    });
};
exports.waitForkWS2PDisconnection = async (server, pubkey) => {
    await new Promise(res => {
        server.pipe(es.mapSync((e) => {
            if (e.ws2p === 'disconnected' && e.peer.pub === pubkey) {
                res();
            }
            return e;
        }));
    });
};
exports.waitForHeads = async (server, nbHeads) => {
    return new Promise(res => {
        server.pipe(es.mapSync((e) => {
            if (e.ws2p === 'heads') {
                if (e.added.length === nbHeads) {
                    res(e.added);
                }
            }
            return e;
        }));
    });
};
class TestingServer {
    constructor(port, server) {
        this.port = port;
        this.server = server;
        server.addEndpointsDefinitions(async () => {
            return require('../../../app/modules/bma').BmaDependency.duniter.methods.getMainEndpoint(server.conf);
        });
    }
    get _server() {
        return this.server;
    }
    get BlockchainService() {
        return this.server.BlockchainService;
    }
    get PeeringService() {
        return this.server.PeeringService;
    }
    get conf() {
        return this.server.conf;
    }
    get dal() {
        return this.server.dal;
    }
    get logger() {
        return this.server.logger;
    }
    get home() {
        return this.server.home;
    }
    revert() {
        return this.server.revert();
    }
    resetHome() {
        return this.server.resetHome();
    }
    on(event, f) {
        return this.server.on(event, f);
    }
    recomputeSelfPeer() {
        return this.server.recomputeSelfPeer();
    }
    async writeBlock(obj) {
        return this.server.writeBlock(obj);
    }
    async writeRawBlock(raw) {
        return this.server.writeRawBlock(raw);
    }
    async writeIdentity(obj) {
        return this.server.writeIdentity(obj);
    }
    async writeCertification(obj) {
        return this.server.writeCertification(obj);
    }
    async writeMembership(obj) {
        return this.server.writeMembership(obj);
    }
    async writeRevocation(obj) {
        return this.server.writeRevocation(obj);
    }
    async writeTransaction(obj) {
        return this.server.writeTransaction(obj);
    }
    async writePeer(obj) {
        return this.server.writePeer(obj);
    }
    async pullingEvent(type, number) {
        this.server.pullingEvent(type, number);
    }
    exportAllDataAsZIP() {
        return this.server.exportAllDataAsZIP();
    }
    unplugFileSystem() {
        return this.server.unplugFileSystem();
    }
    importAllDataFromZIP(zipFile) {
        return this.server.importAllDataFromZIP(zipFile);
    }
    push(chunk, encoding) {
        return this.server.push(chunk, encoding);
    }
    pipe(writable) {
        return this.server.pipe(writable);
    }
    async initDalBmaConnections() {
        await this.server.initWithDAL();
        const bmapi = await bma(this.server);
        this.bma = bmapi;
        const res = await bmapi.openConnections();
        return res;
    }
    url(uri) {
        return 'http://' + [HOST, this.port].join(':') + uri;
    }
    get(uri) {
        return rp(this.url(uri), { json: true });
    }
    post(uri, obj) {
        return rp(this.url(uri), { method: 'POST', json: true, body: obj });
    }
    expect(uri, expectations) {
        return typeof expectations == 'function' ? httpTest.expectAnswer(rp(this.url(uri), { json: true }), expectations) : httpTest.expectJSON(rp(this.url(uri), { json: true }), expectations);
    }
    expectThat(uri, expectations) {
        return httpTest.expectAnswer(rp(this.url(uri), { json: true }), expectations);
    }
    expectJSON(uri, expectations) {
        return httpTest.expectJSON(rp(this.url(uri), { json: true }), expectations);
    }
    expectError(uri, code, message) {
        return httpTest.expectError(code, message, rp(this.url(uri), { json: true }));
    }
    syncFrom(otherServer, fromIncuded, toIncluded) {
        return sync(fromIncuded, toIncluded, otherServer, this.server);
    }
    until(type, count) {
        return until(this.server, type, count);
    }
    async commit(options = null, noWait = false) {
        const raw = await commit(this.server, null, noWait)(options);
        return JSON.parse(raw);
    }
    async commitWaitError(options, expectedError) {
        const results = await Promise.all([
            new Promise(res => {
                this.server.pipe(es.mapSync((e) => {
                    if (e.blockResolutionError === expectedError) {
                        res();
                    }
                }));
            }),
            (async () => {
                const raw = await commit(this.server, null, true)(options);
                return JSON.parse(raw);
            })()
        ]);
        return results[1];
    }
    async commitExpectError(options) {
        try {
            const raw = await commit(this.server)(options);
            JSON.parse(raw);
            throw { message: 'Commit operation should have thrown an error' };
        }
        catch (e) {
            if (e.statusCode) {
                throw JSON.parse(e.error);
            }
        }
    }
    async lookup2identity(search) {
        const lookup = await this.get('/wot/lookup/' + search);
        return IdentityDTO_1.IdentityDTO.fromJSONObject({
            issuer: lookup.results[0].pubkey,
            currency: this.server.conf.currency,
            uid: lookup.results[0].uids[0].uid,
            buid: lookup.results[0].uids[0].meta.timestamp,
            sig: lookup.results[0].uids[0].self
        });
    }
    async readBlock(number) {
        const block = await this.get('/blockchain/block/' + number);
        return BlockDTO_1.BlockDTO.fromJSONObject(block);
    }
    async makeNext(overrideProps) {
        const block = await require('../../../app/modules/prover').ProverDependency.duniter.methods.generateAndProveTheNext(this.server, null, null, overrideProps || {});
        return BlockDTO_1.BlockDTO.fromJSONObject(block);
    }
    async sharePeeringWith(otherServer) {
        let p = await this.get('/network/peering');
        await otherServer.post('/network/peering/peers', {
            peer: PeerDTO_1.PeerDTO.fromJSONObject(p).getRawSigned()
        });
    }
    async getPeer() {
        return this.get('/network/peering');
    }
    waitToHaveBlock(number) {
        return exports.waitToHaveBlock(this.server, number);
    }
    waitForHeads(nbHeads) {
        return exports.waitForHeads(this.server, nbHeads);
    }
    waitForkResolution(number) {
        return exports.waitForkResolution(this.server, number);
    }
    postIdentity(idty) {
        return this.post('/wot/add', {
            identity: idty.getRawSigned()
        });
    }
    postCert(cert) {
        return this.post('/wot/certify', {
            cert: cert.getRawSigned()
        });
    }
    postMembership(ms) {
        return this.post('/blockchain/membership', {
            membership: ms.getRawSigned()
        });
    }
    postRevocation(rev) {
        return this.post('/wot/revoke', {
            revocation: rev.getRaw()
        });
    }
    postBlock(block) {
        return this.post('/blockchain/block', {
            block: block.getRawSigned()
        });
    }
    postRawTX(rawTX) {
        return this.post('/tx/process', {
            transaction: rawTX
        });
    }
    postPeer(peer) {
        return this.post('/network/peering/peers', {
            peer: peer.getRawSigned()
        });
    }
    async prepareForNetwork() {
        await this.server.initWithDAL();
        const bmaAPI = await bma(this.server);
        await bmaAPI.openConnections();
        this.bma = bmaAPI;
        router_1.RouterDependency.duniter.methods.routeToNetwork(this.server);
        // Extra: for /wot/requirements URL
        require('../../../app/modules/prover').ProverDependency.duniter.methods.hookServer(this.server);
    }
    startBlockComputation() {
        if (!this.prover) {
            this.prover = require('../../../app/modules/prover').ProverDependency.duniter.methods.prover(this.server);
            this.permaProver = this.prover.permaProver;
            this.server.pipe(this.prover);
        }
        this.prover.startService();
    }
    // server.startBlockComputation = () => this.prover.startService();
    stopBlockComputation() {
        return this.prover.stopService();
    }
    async closeCluster() {
        const server = this.server;
        if (server._utProver) {
            const farm = await server._utProver.getWorker();
            await farm.shutDownEngine();
        }
    }
}
exports.TestingServer = TestingServer;
async function newWS2PBidirectionnalConnection(currency, k1, k2, serverHandler) {
    let i = 1;
    let port = PORT++;
    const wss = new WebSocketServer({ port });
    let s1;
    let c1;
    return await new Promise(resolveBefore => {
        wss.on('connection', async (ws) => {
            switch (i) {
                case 1:
                    s1 = WS2PConnection_1.WS2PConnection.newConnectionFromWebSocketServer(ws, serverHandler, new WS2PConnection_1.WS2PPubkeyLocalAuth(currency, k1, ""), new WS2PConnection_1.WS2PPubkeyRemoteAuth(currency, k1), {
                        connectionTimeout: 100,
                        requestTimeout: 100
                    });
                    s1.connect().catch((e) => console.error('WS2P: newConnectionFromWebSocketServer connection error'));
                    break;
            }
            resolveBefore({
                p1: s1,
                p2: c1,
                wss
            });
            i++;
        });
        c1 = WS2PConnection_1.WS2PConnection.newConnectionToAddress(1, 'ws://localhost:' + port, new (class EmptyHandler {
            async handlePushMessage(json) {
            }
            async answerToRequest(json) {
                return {};
            }
        }), new WS2PConnection_1.WS2PPubkeyLocalAuth(currency, k2, ""), new WS2PConnection_1.WS2PPubkeyRemoteAuth(currency, k2));
    });
}
exports.newWS2PBidirectionnalConnection = newWS2PBidirectionnalConnection;
exports.simpleWS2PNetwork = async (s1, s2) => {
    let port = exports.getNewTestingPort();
    const clientPub = s2.conf.pair.pub;
    let w1;
    const cluster1 = WS2PCluster_1.WS2PCluster.plugOn(s1._server);
    const cluster2 = WS2PCluster_1.WS2PCluster.plugOn(s2._server);
    const ws2ps = await cluster1.listen('localhost', port);
    const connexionPromise = new Promise(res => {
        ws2ps.on('newConnection', res);
    });
    const ws2pc = await cluster2.connectToRemoteWS(1, 'localhost', port, '', new WS2PServerMessageHandler_1.WS2PServerMessageHandler(s2._server, cluster2), s1._server.conf.pair.pub);
    await connexionPromise;
    w1 = await ws2ps.getConnection(clientPub);
    if (!w1) {
        throw "Connection coming from " + clientPub + " was not found";
    }
    return {
        w1,
        ws2pc,
        wss: ws2ps,
        cluster1,
        cluster2
    };
};
function simpleTestingConf(now = 1500000000, pair) {
    return {
        bmaWithCrawler: true,
        pair,
        nbCores: 1,
        udTime0: now,
        udReevalTime0: now,
        sigQty: 1,
        medianTimeBlocks: 1 // The medianTime always equals previous block's medianTime
    };
}
exports.simpleTestingConf = simpleTestingConf;
//# sourceMappingURL=toolbox.js.map